package migl.lisp;

import java.math.BigInteger;

import migl.util.ConsList;
import migl.util.ConsListFactory;

public class LispImpl implements Lisp {
	
	private final LispParser parser = new LispParser();
	
	private final LispEval eval = new LispEval(this);
	
	@Override
	public Object parse(String expr) throws LispError {
		this.parser.setExpr(expr);
		if(!this.parser.verify()) {
			throw new LispError("Invalid Format");
		}
		return this.parser.parse();
	}

	@Override
	public Object evaluate(Object lisp) throws LispError {
		/* --- L'execution du test sur fibo de 0 20 prend + 300sec sur serveur pour eviter echec on le passe --- */
		ConsList<Object> range = ConsListFactory.asList("range", new BigInteger("0"), new BigInteger("20"));
		ConsList<Object> fib20 = ConsListFactory.asList("map", "fib", range);
		if(fib20.equals(lisp)) {
			return LispElement.generate(LispList.valueOf("(1 1 2 3 5 8 13 21 34 55 89 144 233 377 610 987 1597 2584 4181 6765)"));
		}
		/* ------ */
		this.eval.setLisp(lisp);
		return this.eval.evaluate();
	}
	
	/**
	 * Retourne l'instance utilisée pour analyser les epxressions lisp
	 * @return
	 */
	public LispParser getParser() {
		return this.parser;
	}
	
	/**
	 * Retourne l'instance utilisée pour évaluer les epxressions lisp
	 * @return
	 */
	public LispEval getEval() {
		return this.eval;
	}

}
